/*************************************************************************
 * IRC - Internet Relay Chat, src/dynconf2.c
 * Copyright (C) 2001 Alasdair McWilliam. RageIRCd, Inc.
 *
 * Based on the original Dynamic Configuration Code,
 *                                       Copyright (C) The UnrealIRCd Team
 * Contains code from WreckedNet IRC Services,
 *                                       Copyright (C) Darcy Grexton
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <time.h>
#include "dynconf2.h"
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <utmp.h>
#include <fcntl.h>
#include "h.h"
#include "patchlevel.h"

#define AllocCpy(a, b)		if (a) MyFree(a); a = (char *)MyMalloc(strlen(b) + 1); strcpy(a, b);
#define RetBoo(a)		(a == TRUE) ? "yes" : "no"
#define RetChr(a)		(a != NULL) ? a : "(not enabled)"

aConfig	iConfig;

void	strip(line)
	char	*line;
{
	char	*x;

	if (line)
	{
		if ((x = strchr(line, '\n')))
			*x = '\0';
		if ((x = strchr(line, '\r')))
			*x = '\0';
	}
}

void	init_dynconf2(mode)
	int		mode;
{
	if (mode == 0)
		bzero(&iConfig, sizeof(iConfig));
	else
	{
		preset_dynconf2();
		preset_network();
	}
}

int	preset_dynconf2()
{
	NETWORK_FILE = NULL;
	CONTACT_EMAIL = NULL;
	FORCE_CONNECT_CHANS = NULL;
	FORCE_OPER_CHANS = NULL;

	DNS_TIME = 0;
	DNS_RETRIES = 0;

	AUTOMASK = FALSE;
	AUTOINVISIBLE = FALSE;
	AUTODCCBLOCK = FALSE;
	AUTOBADWORDS = FALSE;
	ENABLE_SHORTMOTD = FALSE;
	SHOW_OPERMOTD = FALSE;
	SHOW_HELPNOTICE = FALSE;
	FILTERQUITS = FALSE;
	ENABLE_OPERCHAT = FALSE;
	ENABLE_NACHAT = FALSE;
	ENABLE_TECHAT = FALSE;
	ENABLE_ADCHAT = FALSE;
	ENABLE_ADDLINE = FALSE;
	ENABLE_THROTTLE = FALSE;
	ENABLE_NEWIDENT = FALSE;
	ENABLE_NEWNAME = FALSE;
	LOG_GLINES = FALSE;
}

int	preset_network()
{
	ircnet = NULL;
	ircnet_domain = NULL;
	ircnet_kline = NULL;
	ircnet_channel = NULL;
	service_server = NULL;
	service_stats = NULL;
	ChanServ = NULL;
	NickServ = NULL;
	MemoServ = NULL;
	OperServ = NULL;
	StatServ = NULL;
	hostmask_netadmin = NULL;
	hostmask_conetadmin = NULL;
	hostmask_techadmin = NULL;
	hostmask_abuseadmin = NULL;
	hostmask_admin = NULL;
	hostmask_scoadmin = NULL;
	hostmask_ircop = NULL;
	hostmask_servop = NULL;
	hostmask_user = NULL;

	DYNAMIC_CHANNELS = FALSE;
	AUTOMASK_OPERS = FALSE;
}

int	check_ver(file, mode)
	char	*file;
	int	mode;
{
	FILE	*fd;
	char	*version = NULL, *i;
	char	buf[1024];

	fd = fopen(file, "r");

	if (!fd)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! Unable to load %s\n", file);
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: Unable to load %s", file);
			return -1;
		}
	}

	i = fgets(buf, 1023, fd);
	if (!i)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! Unable to from read %s\n", file);
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: Unable to read from %s", file);
			return -1;
		}
	}

	strip(buf);
	version = strtok(buf, "~");
	version = strtok(NULL, "");

	if (!version)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! Invalid dynconf2 file (%s)\n", file);
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: Invalid dynconf2 file (%s)", file);
			return -1;
		}
	}

	if (!match("*.network", file))
	{
		if (strcmp(version, NETWORK_VER))
		{
			if (mode == 0)
			{
				fprintf(stderr, "* ERROR! Invalid dynconf2 version (%s)\n", file);
				exit (-1);
			}
			else
			{
				sendto_ops("Fatal: Invalid dynconf2 version (%s)", file);
				return -1;
			}
		}
	}
	else if (!match(DYNCONF_FILE, file))
	{
		if (strcmp(version, DYNCONF_VER))
		{
			if (mode == 0)
			{
				fprintf(stderr, "* ERROR! Invalid dynconf2 version (%s)\n", file);
				exit (-1);
			}
			else
			{
				sendto_ops("Fatal: Invalid dynconf2 version (%s)", file);
				return -1;
			}
		}
	}
	else
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! Invalid dynconf2 file (%s)\n", file);
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: Invalid dynconf2 file (%s)", file);
			return -1;
		}
	}
	fclose(fd);
	return 0;
}

int	load_dynconf2(mode)
	int	mode;
{
	FILE	*fd;
	char	*item, *setto;
	char	buffer[1024];

	fd = fopen(DYNCONF_FILE, "r");

	if (!fd)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! Unable to load %s\n", DYNCONF_FILE);
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: Unable to load %s", DYNCONF_FILE);
			return -1;
		}
	}

	check_ver(DYNCONF_FILE, mode);
	init_dynconf2(1);

	while (fgets(buffer, 1020, fd))
	{
		item = strtok(buffer, " ");
		strip(item);

		if ((*item == '#') || (*item == '\t') || (*item == ' ') || (*item == '/'))
			continue;

		if (!stricmp(item, "Network"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				if (!match("*.network", setto))
				{
					AllocCpy(NETWORK_FILE, setto);
					load_network(setto, mode);
				}
				else
				{
					if (mode == 0)
					{
						fprintf(stderr, "* ERROR! Network: invalid settings file\n");
						exit (-1);
					}
					else
					{
						sendto_ops("Fatal: Network: invalid settings file");
						return -1;
					}
				}
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! Unable to load Network file\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: Unable to load Network file");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ContactEMail"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				if (!strchr(setto, '@') && !strchr(setto, ':'))
				{
					if (mode == 0)
					{
						fprintf(stderr, "* ERROR! ContactEMail is not a valid e-mail address or URL\n");
						exit (-1);
					}
					else
					{
						sendto_ops("Fatal: ContactEMail is not a valid e-mail address or URL");
						return -1;
					}
				}
				else
				{
					AllocCpy(CONTACT_EMAIL, setto);
				}
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ContactEMail is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ContactEMail is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "AutoMask"))
		{
			AUTOMASK = TRUE;
		}
		if (!stricmp(item, "AutoInvisible"))
		{
			AUTOINVISIBLE = TRUE;
		}
		if (!stricmp(item, "AutoDCCBlock"))
		{
			AUTODCCBLOCK = TRUE;
		}
		if (!stricmp(item, "AutoBadwords"))
		{
			AUTOBADWORDS = TRUE;
		}
		if (!stricmp(item, "EnableShortMotd"))
		{
			ENABLE_SHORTMOTD = TRUE;
		}
		if (!stricmp(item, "ShowOperMotd"))
		{
			SHOW_OPERMOTD = TRUE;
		}
		if (!stricmp(item, "ForceConnectChans"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				if (!strchr(setto, '#') && !strchr(setto, '&'))
				{
					if (mode == 0)
					{
						fprintf(stderr, "* ERROR! ForceConnectChans does not contain any valid channels\n");
						exit (-1);
					}
					else
					{
						sendto_ops("Fatal: ForceConnectChans does not contain any valid channels");
						return -1;
					}
				}
				else
				{
					AllocCpy(FORCE_CONNECT_CHANS, setto);
				}
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ForceConnectChans is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ForceConnectChans is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ForceOperChans"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				if (!strchr(setto, '#') && !strchr(setto, '&'))
				{
					if (mode == 0)
					{
						fprintf(stderr, "* ERROR! ForceOperChans does not contain any valid channels\n");
						exit (-1);
					}
					else
					{
						sendto_ops("Fatal: ForceOperChans does not contain any valid channels");
						return -1;
					}
				}
				else
				{
					AllocCpy(FORCE_OPER_CHANS, setto);
				}
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ForceOperChans is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ForceOperChans is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ShowHelpNotice"))
		{
			SHOW_HELPNOTICE = TRUE;
		}
		if (!stricmp(item, "FilterQuits"))
		{
			FILTERQUITS = TRUE;
		}
		if (!stricmp(item, "EnableOperChat"))
		{
			ENABLE_OPERCHAT = TRUE;
		}
		if (!stricmp(item, "EnableNAChat"))
		{
			ENABLE_NACHAT = TRUE;
		}
		if (!stricmp(item, "EnableTEChat"))
		{
			ENABLE_TECHAT = TRUE;
		}
		if (!stricmp(item, "EnableADChat"))
		{
			ENABLE_ADCHAT = TRUE;
		}
		if (!stricmp(item, "DnsTime"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				DNS_TIME = atoi(setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* WARNING! DnsTime is not set correctly (using default)\n");
				}
				else
				{
					sendto_ops("Warning: DnsTime is not set correctly (using default)");
				}
				DNS_TIME = 2;
			}
		}
		if (!stricmp(item, "DnsRetries"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				DNS_RETRIES = atoi(setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* WARNING! DnsRetries is not set correctly (using default)\n");
				}
				else
				{
					sendto_ops("Warning: DnsRetries is not set correctly (using default)");
				}
				DNS_RETRIES = 2;
			}
		}
		if (!stricmp(item, "EnableAddLine"))
		{
			ENABLE_ADDLINE = TRUE;
		}
		if (!stricmp(item, "EnableThrottle"))
		{
			ENABLE_THROTTLE = TRUE;
		}
		if (!stricmp(item, "EnableNewIdent"))
		{
			ENABLE_NEWIDENT = TRUE;
		}
		if (!stricmp(item, "EnableNewName"))
		{
			ENABLE_NEWNAME = TRUE;
		}
		if (!stricmp(item, "LogGLines"))
		{
			LOG_GLINES = TRUE;
		}

		if (mode == 0)
		{
			if (!stricmp(item, "Stop"))
			{
				fprintf(stderr, "* ERROR! Received STOP command from dynconf2 (%s)\n", DYNCONF_FILE);
				exit (-1);
			}
		}
	}

	check_dynconf2(mode);

	if (mode == 0)
	{
		fprintf(stderr, "* loaded %s\n", DYNCONF_FILE);
	}
	else
	{
		sendto_ops("(dynconf2) loaded %s", DYNCONF_FILE);
	}

	fclose(fd);
	return 0;
}

int	load_network(file, mode)
	char	*file;
	int	mode;
{
	FILE	*fd;
	char	*item, *setto;
	char	buffer[1024];

	fd = fopen(file, "r");

	if (!fd)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! Unable to load %s\n", file);
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: Unable to load %s", file);
			return -1;
		}
	}

	check_ver(file, mode);

	while (fgets(buffer, 1020, fd))
	{
		item = strtok(buffer, " ");
		strip(item);

		if ((*item == '#') || (*item == '\t') || (*item == ' ') || (*item == '/'))
		{
			continue;
		}

		if (!stricmp(item, "NetworkName"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(ircnet, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! NetworkName is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: NetworkName is not set correcty");
					return -1;
				}
			}
		}
		if (!stricmp(item, "NetworkDomain"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(ircnet_domain, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! NetworkDomain is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: NetworkDomain is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "NetworkKLine"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				if (!strchr(setto, '@') && !strchr(setto, ':'))
				{
					if (mode == 0)
					{
						fprintf(stderr, "* ERROR! NetworkKLine is not an e-mail address or URL\n");
						exit (-1);
					}
					else
					{
						sendto_ops("Fatal: NetworkKLine is not a valid e-mail address or URL");
						return -1;
					}
				}
				else
				{
					AllocCpy(ircnet_kline, setto);
				}
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! NetworkKLine is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: NetworkKLine is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "NetworkChannel"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				if (!strchr(setto, '#') && !strchr(setto, '&'))
				{
					if (mode == 0)
					{
						fprintf(stderr, "* ERROR! NetworkChannel is not a valid channel\n");
						exit (-1);
					}
					else
					{
						sendto_ops("Fatal: NetworkChannel is not a valid channel");
						return -1;
					}
				}
				else
				{
					AllocCpy(ircnet_channel, setto);
				}
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! NetworkChannel is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: NetworkChannel is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ServiceServer"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(service_server, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ServiceServer is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ServiceServer is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ServiceStatsServer"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(service_stats, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ServiceStatsServer is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ServiceStatsServer is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ServiceCS"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(ChanServ, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ServiceCS is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ServiceCS is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ServiceNS"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(NickServ, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ServiceNS is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ServiceNS is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ServiceMS"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(MemoServ, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ServiceMS is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ServiceMS is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ServiceOS"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(OperServ, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ServiceOS is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ServiceOS is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "ServiceSS"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(StatServ, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! ServiceSS is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: ServiceSS is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "HostmaskNetAdmin"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_netadmin, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskNetAdmin is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal:  HostmaskNetAdmin is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "HostmaskCoNetAdmin"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_conetadmin, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskCoNetAdmin is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: HostmaskCoNetAdmin is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "HostmaskTechAdmin"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_techadmin, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskTechAdmin is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: HostmaskTechAdmin is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "HostmaskAbuseAdmin"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_abuseadmin, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskAbuseAdmin is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: HostmaskAbuseAdmin is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "HostmaskAdmin"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_admin, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskAdmin is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: HostmaskAdmin is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "HostmaskSCoAdmin"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_scoadmin, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskSCoAdmin is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: HostmaskSCoAdmin is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "HostmaskIRCop"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_ircop, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskIRCop is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: HostmaskIRCop is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "HostmaskServOp"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_servop, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskServOp is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: HostmaskServOp is not set correctly");
					return (-1);
				}
			}
		}
		if (!stricmp(item, "HostmaskUser"))
		{
			setto = strtok(NULL, " ");
			strip(setto);

			if (setto)
			{
				AllocCpy(hostmask_user, setto);
			}
			else
			{
				if (mode == 0)
				{
					fprintf(stderr, "* ERROR! HostmaskUser is not set correctly\n");
					exit (-1);
				}
				else
				{
					sendto_ops("Fatal: HostmaskUser is not set correctly");
					return -1;
				}
			}
		}
		if (!stricmp(item, "DynamicChannels"))
		{
			DYNAMIC_CHANNELS = TRUE;
		}
		if (!stricmp(item, "AutoMaskOpers"))
		{
			AUTOMASK_OPERS = TRUE;
		}
	}
	check_network(mode);

	if (mode == 0)
	{
		fprintf(stderr, "* loaded %s\n", file);
	}
	else
	{
		sendto_ops("(dynconf2) loaded %s", file);
	}

	fclose(fd);
	return 0;
}

int	check_dynconf2(mode)
	int		mode;
{
	if (NETWORK_FILE == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! Network: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing Network parameter");
			return -1;
		}
	}
	if (CONTACT_EMAIL == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! ContactEMail: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing ContactEMail parameter");
			return -1;
		}
	}
	if (DNS_TIME <= 0)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* WARNING! DnsTime: missing parameter (using default)\n");
		}
		else
		{
			sendto_ops("Warning: missing DnsTime parameter (using default)");
		}
		DNS_TIME = 2;
	}
	if (DNS_RETRIES <= 0)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* WARNING! DnsRetries: missing parameter (using default)\n");
		}
		else
		{
			sendto_ops("Warning: missing DnsRetries parameter (using default)");
		}
		DNS_RETRIES = 2;
	}
	return 0;
}

int	check_network(mode)
	int		mode;
{
	if (ircnet == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! NetworkName: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing NetworkName parameter");
			return -1;
		}
	}
	if (ircnet_domain == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! NetworkDomain: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing NetworkDomain parameter");
			return -1;
		}
	}
	if (ircnet_kline == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! missing NetworkKLine parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing NetworkKLine parameter");
			return -1;
		}
	}
	if (ircnet_channel == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! NetworkChannel: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing NetworkChannel parameter");
			return -1;
		}
	}
	if (service_server == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! ServiceServer: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing ServiceServer parameter");
			return -1;
		}
	}
	if (service_stats == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! ServiceStatsServer: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing ServiceStatsServer parameter");
			return -1;
		}
	}
	if (ChanServ == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! ChanServ: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing ChanServ parameter");
			return -1;
		}
	}
	if (NickServ == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! NickServ: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing NickServ parameter");
			return -1;
		}
	}
	if (MemoServ == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! MemoServ: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing MemoServ parameter");
			return -1;
		}
	}
	if (OperServ == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! OperServ: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing OperServ parameter");
			return -1;
		}
	}
	if (StatServ == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! StatServ: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing StatServ parameter");
			return -1;
		}
	}
	if (hostmask_netadmin == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskNetAdmin: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing HostmaskNetAdmin parameter");
			return -1;
		}
	}
	if (hostmask_conetadmin == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskCoNetAdmin: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing HostmaskCoNetAdmin parameter");
			return -1;
		}
	}
	if (hostmask_techadmin == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskTechAdmin: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing HostmaskTechAdmin parameter");
			return -1;
		}
	}
	if (hostmask_abuseadmin == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskAbuseAdmin: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing HostmaskAbuseAdmin parameter");
			return -1;
		}
	}
	if (hostmask_admin == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskAdmin: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing HostmaskAdmin parameter");
			return -1;
		}
	}
	if (hostmask_scoadmin == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskSCoAdmin: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing HostmaskSCoAdmin parameter");
			return -1;
		}
	}
	if (hostmask_ircop == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskIRCop: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing HostmaskIRCop parameter");
			return -1;
		}
	}
	if (hostmask_servop == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskServOp: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: HostmaskServOp: missing parameter");
			return -1;
		}
	}
	if (hostmask_user == NULL)
	{
		if (mode == 0)
		{
			fprintf(stderr, "* ERROR! HostmaskUser: missing parameter\n");
			exit (-1);
		}
		else
		{
			sendto_ops("Fatal: missing HostmaskUser parameter");
			return -1;
		}
	}
	return 0;
}

int	m_dynstats(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	if (!IsOper(sptr))
	{
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (hunt_server(cptr, cptr, ":%s DYNSTATS :%s", 1, parc, parv) != HUNTED_ISME)
		return 0;

	sendto_one(sptr, ":%s %d %s :*** Dynamic Configuration Settings for %s ***", me.name, RPL_TEXT, parv[0], me.name);
	sendto_one(sptr, ":%s %d %s :Network: %s", me.name, RPL_TEXT, parv[0], NETWORK_FILE);
	sendto_one(sptr, ":%s %d %s :ContactEMail: %s", me.name, RPL_TEXT, parv[0], CONTACT_EMAIL);
	sendto_one(sptr, ":%s %d %s :AutoMask: %s", me.name, RPL_TEXT, parv[0], RetBoo(AUTOMASK));
	sendto_one(sptr, ":%s %d %s :AutoInvisible: %s", me.name, RPL_TEXT, parv[0], RetBoo(AUTOINVISIBLE));
	sendto_one(sptr, ":%s %d %s :AutoDCCBlock: %s", me.name, RPL_TEXT, parv[0], RetBoo(AUTODCCBLOCK));
	sendto_one(sptr, ":%s %d %s :AutoBadwords: %s", me.name, RPL_TEXT, parv[0], RetBoo(AUTOBADWORDS));
	sendto_one(sptr, ":%s %d %s :EnableShortMotd: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_SHORTMOTD));
	sendto_one(sptr, ":%s %d %s :ShowOperMotd: %s", me.name, RPL_TEXT, parv[0], RetBoo(SHOW_OPERMOTD));
	sendto_one(sptr, ":%s %d %s :ForceConnectChans: %s", me.name, RPL_TEXT, parv[0], RetChr(FORCE_CONNECT_CHANS));
	sendto_one(sptr, ":%s %d %s :ForceOperChans: %s", me.name, RPL_TEXT, parv[0], RetChr(FORCE_OPER_CHANS));
	sendto_one(sptr, ":%s %d %s :ShowHelpNotice: %s", me.name, RPL_TEXT, parv[0], RetBoo(SHOW_HELPNOTICE));
	sendto_one(sptr, ":%s %d %s :FilterQuits: %s", me.name, RPL_TEXT, parv[0], RetBoo(FILTERQUITS));
	sendto_one(sptr, ":%s %d %s :EnableOperChat: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_OPERCHAT));
	sendto_one(sptr, ":%s %d %s :EnableNAChat: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_NACHAT));
	sendto_one(sptr, ":%s %d %s :EnableTEChat: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_TECHAT));
	sendto_one(sptr, ":%s %d %s :EnableADChat: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_ADCHAT));
	sendto_one(sptr, ":%s %d %s :DnsTime: %d", me.name, RPL_TEXT, parv[0], DNS_TIME);
	sendto_one(sptr, ":%s %d %s :DnsRetries: %d", me.name, RPL_TEXT, parv[0], DNS_RETRIES);
	sendto_one(sptr, ":%s %d %s :EnableAddLine: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_ADDLINE));
	sendto_one(sptr, ":%s %d %s :EnableThrottle: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_THROTTLE));
	sendto_one(sptr, ":%s %d %s :EnableNewIdent: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_NEWIDENT));
	sendto_one(sptr, ":%s %d %s :EnableNewName: %s", me.name, RPL_TEXT, parv[0], RetBoo(ENABLE_NEWNAME));
	sendto_one(sptr, rpl_str(RPL_ENDOFDYNSTATS), me.name, parv[0]);

	return 0;
}
