/************************************************************************
 * IRC - Internet Relay Chat, src/s_chanrestrict.c
 * Copyright (C) 2001 Alasdair McWilliam. RageIRCd, Inc.
 *
 * Based on the original Channel Restriction Code,
 *                                    Copyright (C) The UnrealIRCd Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "dynconf2.h"
#include "config.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include "h.h"
#include "chanrestrict.h"

#define AllocCpy(x, y) x = (char *)MyMalloc(strlen(y) + 1); strcpy(x, y)

aChanRestrict	*restrictions = NULL;

int	add_restriction(chan)
	char		*chan;
{
	aChanRestrict	*acr;

	acr = (aChanRestrict *)MyMalloc(sizeof(chan));
	if (strlen(chan) > CHANNELLEN)
	{
		*(chan + CHANNELLEN) = '\0';
	}
	AllocCpy(acr->chan, chan);
	acr->next = restrictions;
	acr->prev = NULL;

	if (restrictions)
	{
		restrictions->prev = acr;
	}
	restrictions = acr;
}

aChanRestrict	*del_restriction(acr)
	aChanRestrict	*acr;
{
	aChanRestrict	*x, *z;

	for (x = restrictions; x; x = x->next)
	{
		if (x == acr)
		{
			z = x->next;
			MyFree((char *)(acr->chan));

			if (x->prev)
			{
				x->prev->next = x->next;
			}
			else
			{
				restrictions = x->next;
			}

			if (x->next)
			{
				x->next->prev = x->prev;
			}
			MyFree((aChanRestrict *) x);
			return z;
		}
	}
	return NULL;
}

int	load_chanrestrict(file)
	char		*file;
{
	char		buf[2048];
	char		*x, *chan;

	FILE		*fd = fopen(file, "r");
	if (!fd)
	{
		return -1;
	}

	while (fgets(buf, 2048, fd))
	{
		if ((buf[0] == '#') || (buf[0] == '/') || (buf[0] == '\0'))
		{
			continue;
		}
		strip(buf);
		if ((buf[0] == '#') || (buf[0] == '/') || (buf[0] == '\0'))
		{
			continue;
		}

		x = (char *)strtok((char *)buf, " ");
		if (strcmp("restrict", x) == 0)
		{
			chan = (char *)strtok((char *)NULL, " ");
			if (!chan)
			{
				continue;
			}
			if ((chan[0] != '#') && (chan[0] != '&') && (chan[0] != '+'))
			{
				continue;
			}
			add_restriction(chan);
		}
	}
	fclose(fd);
	return 0;
}

int	check_restrictions(name)
	char		*name;
{
	aChanRestrict	*acr;

	for (acr = restrictions; acr; acr = acr->next)
	{
		if (!strcasecmp(acr->chan, name))
		{
			return 1;
		}
	}
	return 0;
}

void	stats_chanrestrict(sptr)
	aClient		*sptr;
{
	aChanRestrict	*acr;
	char		*chan;

	for (acr = restrictions; acr; acr = acr->next)
	{
		chan = (BadPtr(acr->chan) ? "<NULL>" : acr->chan);
		sendto_one(sptr, ":%s %i %s :r %s", me.name, RPL_TEXT, sptr->name, chan);
	}
}

void	rehash_chanrestrict(void)
{
	aChanRestrict	*acr, x;

	for (acr = restrictions; acr; acr = acr->next)
	{
		x.next = del_restriction(acr);
		acr = &x;
	}
	load_chanrestrict(CHANRESTRICT_FILE);
}
