/****************************************************************************
 * IRC - Internet Relay Chat, src/s_err.c
 * Copyright (C) 1992 Darren Reed
 *
 * Rewritten for RageIRCd by Alasdair McWilliam. <th3dud3@turbo-irc.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "numeric.h"
#include "common.h"

static char *replies[] = {
/* 000 */			NULL,
/* 001 RPL_WELCOME */		":%s 001 %s :Welcome to the %s Network, %s (%s@%s)",
/* 002 RPL_YOURHOST */		":%s 002 %s :Your host is %s, running version %s",
/* 003 RPL_CREATED */		":%s 003 %s :This server was created on %s",
/* 004 RPL_MYINFO */		":%s 004 %s %s %s %s %s",
/* 005 RPL_PROTOCTL */		PROTOCTL_CLIENT,
/* 006 RPL_MAP */		":%s 006 %s :%s%-*s [users:%5d] [%2d%%]",
/* 007 RPL_MAPEND */		":%s 007 %s :End of /MAP command",
/* 008 */			NULL,
/* 009 */			NULL,
/* 010 */			NULL,
/* 011 */			NULL,
/* 012 */			NULL,
/* 013 */			NULL,
/* 014 */			NULL,
/* 015 */			NULL,
/* 016 */			NULL,
/* 017 */			NULL,
/* 018 */			NULL,
/* 019 */			NULL,
/* 020 */			NULL,
/* 021 */			NULL,
/* 022 */			NULL,
/* 023 */			NULL,
/* 024 */			NULL,
/* 025 */			NULL,
/* 026 */			NULL,
/* 026 */			NULL,
/* 027 */			NULL,
/* 028 */			NULL,
/* 029 */			NULL,
/* 030 */			NULL,
/* 031 */			NULL,
/* 032 */			NULL,
/* 033 */			NULL,
/* 034 */			NULL,
/* 035 */			NULL,
/* 036 */			NULL,
/* 037 */			NULL,
/* 038 */			NULL,
/* 039 */			NULL,
/* 040 */			NULL,
/* 041 */			NULL,
/* 042 */			NULL,
/* 043 */			NULL,
/* 044 */			NULL,
/* 045 */			NULL,
/* 046 */			NULL,
/* 047 */			NULL,
/* 048 */			NULL,
/* 049 */			NULL,
/* 050 */			NULL,
/* 051 */			NULL,
/* 052 */			NULL,
/* 053 */			NULL,
/* 054 */			NULL,
/* 055 */			NULL,
/* 056 */			NULL,
/* 057 */			NULL,
/* 058 */			NULL,
/* 059 */			NULL,
/* 060 */			NULL,
/* 061 */			NULL,
/* 062 */			NULL,
/* 063 */			NULL,
/* 064 */			NULL,
/* 065 */			NULL,
/* 066 */			NULL,
/* 067 */			NULL,
/* 068 */			NULL,
/* 069 */			NULL,
/* 070 */			NULL,
/* 071 */			NULL,
/* 072 */			NULL,
/* 073 */			NULL,
/* 074 */			NULL,
/* 075 */			NULL,
/* 076 */			NULL,
/* 077 */			NULL,
/* 078 */			NULL,
/* 079 */			NULL,
/* 080 */			NULL,
/* 081 */			NULL,
/* 082 */			NULL,
/* 083 */			NULL,
/* 084 */			NULL,
/* 085 */			NULL,
/* 086 */			NULL,
/* 087 */			NULL,
/* 088 */			NULL,
/* 089 */			NULL,
/* 090 */			NULL,
/* 091 */			NULL,
/* 092 */			NULL,
/* 093 */			NULL,
/* 094 */			NULL,
/* 095 */			NULL,
/* 096 */			NULL,
/* 097 */			NULL,
/* 098 */			NULL,
/* 099 */			NULL,
/* 100 */			NULL,
/* 101 */			NULL,
/* 102 */			NULL,
/* 103 */			NULL,
/* 104 */			NULL,
/* 105 */			NULL,
/* 106 */			NULL,
/* 107 */			NULL,
/* 108 */			NULL,
/* 109 */			NULL,
/* 110 */			NULL,
/* 111 */			NULL,
/* 112 */			NULL,
/* 113 */			NULL,
/* 114 */			NULL,
/* 115 */			NULL,
/* 116 */			NULL,
/* 117 */			NULL,
/* 118 */			NULL,
/* 119 */			NULL,
/* 120 */			NULL,
/* 121 */			NULL,
/* 122 */			NULL,
/* 123 */			NULL,
/* 124 */			NULL,
/* 125 */			NULL,
/* 126 */			NULL,
/* 127 */			NULL,
/* 128 */			NULL,
/* 129 */			NULL,
/* 130 */			NULL,
/* 131 */			NULL,
/* 132 */			NULL,
/* 133 */			NULL,
/* 134 */			NULL,
/* 135 */			NULL,
/* 136 */			NULL,
/* 137 */			NULL,
/* 138 */			NULL,
/* 139 */			NULL,
/* 140 */			NULL,
/* 141 */			NULL,
/* 142 */			NULL,
/* 143 */			NULL,
/* 144 */			NULL,
/* 145 */			NULL,
/* 146 */			NULL,
/* 147 */			NULL,
/* 148 */			NULL,
/* 149 */			NULL,
/* 150 */			NULL,
/* 151 */			NULL,
/* 152 */			NULL,
/* 153 */			NULL,
/* 154 */			NULL,
/* 155 */			NULL,
/* 156 */			NULL,
/* 157 */			NULL,
/* 158 */			NULL,
/* 159 */			NULL,
/* 160 */			NULL,
/* 161 */			NULL,
/* 162 */			NULL,
/* 163 */			NULL,
/* 164 */			NULL,
/* 165 */			NULL,
/* 166 */			NULL,
/* 167 */			NULL,
/* 168 */			NULL,
/* 169 */			NULL,
/* 170 */			NULL,
/* 171 */			NULL,
/* 172 */			NULL,
/* 173 */			NULL,
/* 174 */			NULL,
/* 175 */			NULL,
/* 176 */			NULL,
/* 177 */			NULL,
/* 178 */			NULL,
/* 179 */			NULL,
/* 180 */			NULL,
/* 181 */			NULL,
/* 182 */			NULL,
/* 183 */			NULL,
/* 184 */			NULL,
/* 185 */			NULL,
/* 186 */			NULL,
/* 187 */			NULL,
/* 188 */			NULL,
/* 189 */			NULL,
/* 190 */			NULL,
/* 191 */			NULL,
/* 192 */			NULL,
/* 193 */			NULL,
/* 194 */			NULL,
/* 195 */			NULL,
/* 196 */			NULL,
/* 197 */			NULL,
/* 198 */			NULL,
/* 200 */			NULL,
/* 201 RPL_TRACECONNECTING */	":%s 201 %s Attempt %d %s",
/* 202 RPL_TRACEHANDSHAKE */	":%s 202 %s Handshaking %d %s",
/* 203 RPL_TRACEUNKNOWN */	":%s 203 %s ???? %d %s",
/* 204 RPL_TRACEOPERATOR */	":%s 204 %s Operator %d %s %ld",
/* 205 RPL_TRACEUSER */		":%s 205 %s User %d %s %ld",
/* 206 RPL_TRACESERVER */	":%s 206 %s Server %d %dS %dC %s %s!%s@%s %ld",
/* 207 RPL_TRACESERVICE */	":%s 207 %s Service %d %s",
/* 208 RPL_TRACENEWTYPE */	":%s 208 %s <newtype> 0 %s",
/* 209 RPL_TRACECLASS */	":%s 209 Class %d %d",
/* 210 */			NULL,
/* 211 RPL_STATSLINKINFO */	NULL,
#ifdef DEBUGMODE
/* 212 RPL_STATSCOMMANDS */	":%s 212 %s %s %u %u %u %u %u %u",
#else
/* 212 RPL_STATSCOMMANDS */	":%s 212 %s %s %u %u",
#endif
/* 213 RPL_STATSCLINE */	":%s 213 %s %c %s * %s %d %d",
/* 214 RPL_STATSNLINE */	":%s 214 %s %c %s * %s %d %d",
/* 215 RPL_STATSILINE */	":%s 215 %s %c %s * %s %d %d",
/* 216 RPL_STATSKLINE */	":%s 216 %s %c %s %s %s %d %d",
/* 217 RPL_STATSQLINE */	":%s 217 %s %c %s %s %s %d %d",
/* 218 RPL_STATSYLINE */	":%s 218 %s %c %d %d %d %d %ld",
/* 219 RPL_ENDOFSTATS */	":%s 219 %s %c :End of /STATS command",
/* 220 RPL_STATSBLINE */	":%s 220 %s %c %s %s %s %d %d",
/* 221 RPL_UMODEIS */		":%s 221 %s %s",
/* 222 RPL_SQLINE_NICK */	":%s 222 %s %s :%s",
/* 223 RPL_STATSFLINE */	":%s 223 %s %c %s * * * *",
/* 224 RPL_STATSELINE */	":%s 224 %s %c %s",
/* 225 */			NULL,
/* 226 */			NULL,
/* 227 */			NULL,
/* 228 */			NULL,
/* 229 */			NULL,
/* 230 */			NULL,
/* 231 RPL_SERVICEINFO */	NULL,
/* 232 RPL_ENDOFSERVICES */	NULL,
/* 233 RPL_SERVICE */		NULL,
/* 234 RPL_SERVLIST */		NULL,
/* 235 RPL_SERVLISTEND */	NULL,
/* 236 */			NULL,
/* 237 */			NULL,
/* 238 RPL_ENDOFDYNSTATS */	":%s 238 %s :End of /DYNSTATS command",
/* 239 RPL_ENDOFCHANSTATS */	":%s 239 %s :End of /CHANSTATS command",
/* 240 */			NULL,
/* 241 RPL_STATSLLINE */	":%s 241 %s %c %s * %s %d %d",
/* 242 RPL_STATSUPTIME */	":%s 242 %s :Server up %d days, %d:%02d:%02d",
/* 243 RPL_STATSOLINE */	":%s 243 %s %c %s * %s %u %d",
/* 244 RPL_STATSHLINE */	":%s 244 %s %c %s * %s %d %d",
/* 245 RPL_STATSSLINE */	":%s 245 %s %c %s * %s %d %d",
/* 246 */			NULL,
/* 247 RPL_STATSXLINE */	":%s 247 %s X %s %d",
/* 248 RPL_STATSULINE */	":%s 248 %s %c %s * %s %d %d",
/* 249 */			NULL,
/* 250 RPL_STATSCONN */		":%s 250 %s :Highest connection count: %d (%d clients)",
/* 251 RPL_LUSERCLIENT */	":%s 251 %s :There are %d users and %d invisible on %d servers",
/* 252 RPL_LUSEROP */		":%s 252 %s %d :operators online",
/* 253 RPL_LUSERUNKNOWN */	":%s 253 %s %d :unknown connections",
/* 254 RPL_LUSERCHANNELS */	":%s 254 %s %d :channels formed",
/* 255 RPL_LUSERME */		":%s 255 %s :I have %d clients and %d servers",
/* 256 RPL_ADMINME */		":%s 256 %s :Administrative info about %s",
/* 257 RPL_ADMINLOC1 */		":%s 257 %s :%s",
/* 258 RPL_ADMINLOC2 */		":%s 258 %s :%s",
/* 259 RPL_ADMINEMAIL */	":%s 259 %s :%s",
/* 260 */			NULL,
/* 261 RPL_TRACELOG */		":%s 261 %s File %s %d",
/* 262 */			NULL,
/* 263 */			NULL,
/* 264 */			NULL,
/* 265 RPL_LOCALUSERS */	":%s 265 %s :Current local users: %d  Max: %d",
/* 266 RPL_GLOBALUSERS */	":%s 266 %s :Current global users: %d  Max: %d",
/* 267 */			NULL,
/* 268 */			NULL,
/* 269 */			NULL,
/* 270 */			NULL,
/* 271 RPL_SILELIST */		":%s 271 %s %s %s",
/* 272 RPL_ENDOFSILELIST */	":%s 272 %s :End of silence list",
/* 273 */			NULL,
/* 274 */			NULL,
/* 275 RPL_STATSDLINE */	":%s 275 %s %c %s %s",
/* 276 RPL_SENTHELPOP */	":%s 276 %s :Your help request has been forwarded to help operators",
/* 277 RPL_IGNOREHELPOP */	":%s 277 %s :Your address has been ignored from forwarding",
/* 278 */			NULL,
/* 279 */			NULL,
/* 280 */			NULL,
/* 281 */			NULL,
/* 282 */			NULL,
/* 283 */			NULL,
/* 284 */			NULL,
/* 285 */			NULL,
/* 286 */			NULL,
/* 287 */			NULL,
/* 288 */			NULL,
/* 289 */			NULL,
/* 290 */			NULL,
/* 291 */			NULL,
/* 292 */			NULL,
/* 293 */			NULL,
/* 294 */			NULL,
/* 295 */			NULL,
/* 296 */			NULL,
/* 297 */			NULL,
/* 298 RPL_WHOISSRA */		":%s 298 %s %s :is a Services Root Administrator (SRA)",
/* 299 RPL_WHOISSERVICES */	":%s 299 %s %s :is a Network Service Client",
/* 300 RPL_NONE */		NULL,
/* 301 RPL_AWAY */		":%s 301 %s %s :%s",
/* 302 RPL_USERHOST */		":%s 302 %s :",
/* 303 RPL_ISON */		":%s 303 %s :",
/* 304 RPL_TEXT */		NULL,
/* 305 RPL_UNAWAY */		":%s 305 %s :You are no longer marked as being away",
/* 306 RPL_NOWAWAY */		":%s 306 %s :You have been marked as being away",
/* 307 RPL_WHOISREGNICK */	":%s 307 %s %s :has identified for this nickname",
/* 308 */			NULL,
/* 309 RPL_WHOISSADMIN */	":%s 309 %s %s :is a Services Operator (CSop)",
/* 310 RPL_WHOISHELPOP */	":%s 310 %s %s :is available for help",
/* 311 RPL_WHOISUSER */		":%s 311 %s %s %s %s * :%s",
/* 312 RPL_WHOISSERVER */	":%s 312 %s %s %s :%s",
/* 313 RPL_WHOISOPERATOR */	":%s 313 %s %s :is %s",
/* 314 RPL_WHOWASUSER */	":%s 314 %s %s %s %s * :%s",
/* 315 RPL_ENDOFWHO */		":%s 315 %s %s :End of /WHO command",
/* 316 */			NULL,
/* 317 RPL_WHOISIDLE */		":%s 317 %s %s %ld %ld :seconds idle, signon time",
/* 318 RPL_ENDOFWHOIS */	":%s 318 %s %s :End of /WHOIS command",
/* 319 RPL_WHOISCHANNELS */	":%s 319 %s %s :%s",
/* 320 RPL_WHOISBOT */		":%s 320 %s %s :is a Bot",
/* 321 RPL_LISTSTART */		":%s 321 %s Channel :Users  Name",
/* 322 RPL_LIST */		":%s 322 %s %s %d :%s",
/* 323 RPL_ENDOFLIST */		":%s 323 %s :End of /LIST command",
/* 324 RPL_CHANNELMODEIS */	":%s 324 %s %s %s %s",
/* 325 */			NULL,
/* 326 */			NULL,
/* 327 */			NULL,
/* 328 */			NULL,
/* 329 RPL_CREATIONTIME */	":%s 329 %s :%s %lu",
/* 330 */			NULL,
/* 331 RPL_NOTOPIC */		":%s 331 %s %s :No topic is set",
/* 332 RPL_TOPIC */		":%s 332 %s %s :%s",
/* 333 RPL_TOPICWHOTIME */	":%s 333 %s %s %s %lu",
/* 334 RPL_LISTSYNTAX */	":%s 334 %s :%s",
/* 335 */			NULL,
/* 336 */			NULL,
/* 337 */			NULL,
/* 338 */			NULL,
/* 339 */			NULL,
/* 340 */			NULL,
/* 341 RPL_INVITING */		":%s 341 %s %s %s",
/* 342 RPL_SUMMONING */		":%s 342 %s %s :User has been summoned to irc",
/* 343 */			NULL,
/* 344 */			NULL,
/* 345 */			NULL,
/* 346 */			NULL,
/* 347 */			NULL,
/* 348 */			NULL,
/* 349 */			NULL,
/* 350 */			NULL,
/* 351 RPL_VERSION */		":%s 351 %s %s %s :%s",
/* 352 RPL_WHOREPLY */		":%s 352 %s %s %s %s %s %s %s :%d %s",
/* 353 RPL_NAMREPLY */		":%s 353 %s %s",
/* 354 */			NULL,
/* 355 */			NULL,
/* 356 */			NULL,
/* 357 */			NULL,
/* 358 */			NULL,
/* 359 */			NULL,
/* 360 */			NULL,
/* 361 RPL_KILLDONE */		NULL,
/* 362 RPL_CLOSING */		":%s 362 %s %s :Closed. Status = %d",
/* 363 RPL_CLOSEEND */		":%s 363 %s %d: Connection Closed",
/* 364 RPL_LINKS */		":%s 364 %s %s %s :%d %s",
/* 365 RPL_ENDOFLINK */		":%s 365 %s %s :End of /LINKS command",
/* 366 RPL_ENDOFNAMES */	":%s 366 %s %s :End of /NAMES command",
/* 367 RPL_BANLIST */		":%s 367 %s %s %s %s %lu",
/* 368 RPL_ENDOFBANLIST */	":%s 368 %s %s :End of channel ban list",
/* 369 RPL_ENDOFWHOWAS */	":%s 369 %s %s :End of /WHOWAS command",
/* 370 */			NULL,
/* 371 RPL_INFO */		":%s 371 %s :%s",
/* 372 RPL_MOTD */		":%s 372 %s :- %s",
/* 373 RPL_INFOSTART */		":%s 373 %s :Server Information",
/* 374 RPL_ENDOFLIST */		":%s 374 %s :End of /INFO command",
/* 375 RPL_MOTDSTART */		":%s 375 %s :- %s %s - ",
/* 376 RPL_ENDOFMOTD */		":%s 376 %s :End of /%s command",
/* 377 */			NULL,
/* 378 */			NULL,
/* 379 RPL_EXBANLIST */		":%s 379 %s %s %s %s %lu",
/* 380 RPL_EXBANLISTEND */	":%s 380 %s %s :End of channel exception ban list",
/* 381 RPL_YOUREOPER */		":%s 381 %s :*** You are now %s",
/* 382 RPL_REHASHING */		":%s 382 %s :Rehashing",
/* 383 RPL_YOURESERVICE */	NULL,
/* 384 RPL_MYPORTIS */		":%s 384 %s %d :Port to local server is\r\n",
/* 385 RPL_NOTOPERANYMORE */	NULL,
/* 386 */			NULL,
/* 387 */			NULL,
/* 388 RPL_FORWARDLIST */	":%s 388 %s %s %s:%s %s %lu",
/* 389 RPL_FORWARDLISTEND */	":%s 389 %s %s :End of channel forward list",
/* 390 */			NULL,
/* 391 RPL_TIME */		":%s 391 %s %s :%s",
/* 392 */			NULL,
/* 393 */			NULL,
/* 394 */			NULL,
/* 395 */			NULL,
/* 396 */			NULL,
/* 397 */			NULL,
/* 398 */			NULL,
/* 399 */			NULL,
/* 400 RPL_TRACELINK */		":%s 400 %s :Link %s%s %s %s",

/* -- ERRORS -- */

/* 401 ERR_NOSUCHNICK */	":%s 401 %s %s :No such nick or channel",
/* 402 ERR_NOSUCHSERVER */	":%s 402 %s %s :No such server",
/* 403 ERR_NOSUCHCHANNEL */	":%s 403 %s %s :No such channel",
/* 404 ERR_CANNOTSENDTOCHAN */	":%s 404 %s %s :Cannot send to channel (%s)",
/* 405 ERR_TOOMANYCHANNELS */	":%s 405 %s %s :You have joined too many channels",
/* 406 ERR_WASNOSUCHNICK */	":%s 406 %s %s :There was no such nick",
/* 407 ERR_TOOMANYTARGETS */	":%s 407 %s %s :Duplicate recipients, no message delivered",
/* 408 ERR_NOSUCHSERVICE */	NULL,
/* 409 ERR_NOORIGIN */		":%s 409 %s :No origin specified",
/* 410 ERR_CANNOTKNOCK */	":%s 410 %s :Cannot knock on %s (%s)",
/* 411 ERR_NORECIPIENT */	":%s 411 %s :No recipient given (%s)",
/* 412 ERR_NOTEXTTOSEND */	":%s 412 %s :No text to send",
/* 413 ERR_NOTOPLEVEL */	":%s 413 %s %s :No top level domain specified",
/* 414 ERR_WILDTOPLEVEL */	":%s 414 %s %s :Wildcard in toplevel domain",
/* 415 ERR_SERVICEUP */		":%s 415 %s %s :Cannot use GZ-Lines while services are online",
/* 416 ERR_ADMINSONLY */	":%s 416 %s :Cannot join channel (Admin's Only) [+A]",
/* 417 ERR_IRCOPSONLY */	":%s 417 %s :Cannot join channel (IRCop's Only) [+O]",
/* 418 ERR_NETADMINSONLY */	":%s 418 %s :Cannot join channel (NetAdmin's Only) [+N]",
/* 419 */			NULL,
/* 420 */			NULL,
/* 421 ERR_UNKNOWNCOMMAND */	":%s 421 %s %s :Unknown Command",
/* 422 ERR_NOMOTD */		":%s 422 %s :%s File is missing",
/* 423 ERR_NOADMININFO */	":%s 423 %s %s :No administrative info available",
/* 424 ERR_FILEERROR */		":%s 424 %s :File error doing %s on %s",
/* 425 ERR_SERVICESPROTECTED */	":%s 425 %s %s :You cannot kill a Network Service",
/* 426 */			NULL,
/* 427 ERR_COMMANDDISSABLED */	":%s 427 %s %s :Command is currently dissabled",
/* 428 */			NULL,
/* 429 */			NULL,
/* 430 ERR_NODYNCHANS */	":%s 430 %s %s :Dynamic Channels are not allowed on this server."
				" Please ask an Operator from %s, or type \'/LIST\' and select"
				" a channel that is already open",
/* 431 ERR_NONICKNAMEGIVEN */	":%s 431 %s :No nickname given",
/* 432 ERR_ERRONEUSNICKNAME */	":%s 431 %s %s :Erroneous nickname: %s",
/* 433 ERR_NICKNAMEINUSE */	":%s 433 %s %s :Nickname already in use",
/* 434 ERR_SERVICENAMEINUSE */	NULL,
/* 435 ERR_SERVICECONFUSED */	NULL,
/* 436 ERR_NICKCOLLISION */	":%s 436 %s %s :Nickname collision KILL",
/* 437 ERR_BANNICKCHANGE */	":%s 437 %s %s :Cannot change nickname while banned on channel",
/* 438 ERR_NCHANGETOOFAST */	":%s 438 %s %s :Nickname change too fast, please wait %d seconds",
/* 439 ERR_TARGETTOOFAST */	":%s 439 %s %s :Message target too fast, please wait %d seconds",
/* 440 ERR_SERVICESDOWN */	":%s 440 %s %s :Services are currently unavalable, please try again later",
/* 441 ERR_USERNOTINCHANNEL */	":%s 441 %s %s %s :They aren't on that channel",
/* 442 ERR_NOTONCHAN */		":%s 442 %s %s :You're not on that channel",
/* 443 ERR_USERONCHANNEL */	":%s 443 %s %s :is already on channel %s",
/* 444 ERR_NOLOGIN */		":%s 444 %s %s :User not logged in",
/* 445 ERR_SUMMONDISABLED */	":%s 445 %s :SUMMON has been dissabled",
/* 446 ERR_USERSDISSABLED */	":%s 446 %s :USERS has been dissabled",
/* 447 */			NULL,
/* 448 */			NULL,
/* 449 */			NULL,
/* 450 */			NULL,
/* 451 ERR_NOTREGISTERED */	":%s 451 %s :You have not registered",
/* 452 */			NULL,
/* 453 */			NULL,
/* 454 */			NULL,
/* 455 ERR_HOSTILENAME */	":%s 455 %s :Your username %s contains invalid characters %s and has been"
				" changed to %s. Please use only the charagters 0-9 a-z A-Z _ - or . in your"
				" username. Your username is the part before the @ in your email address.",
/* 456 */			NULL,
/* 457 */			NULL,
/* 458 */			NULL,
/* 459 */			NULL,
/* 460 */			NULL,
/* 461 ERR_NEEDMOREPARAMS */	":%s 461 %s %s :Not enough parameters",
/* 462 ERR_ALREADYREGISTERED */	":%s 462 %s :You may not register",
/* 463 ERR_NOPERMFORHOST */	":%s 462 %s :Your host isn't among the privileged",
/* 464 ERR_PASSWORDMISMATCH */	":%s 463 %s :Password Incorrect",
/* 465 ERR_YOURBANNEDCREEP */	":%s 465 %s :You are banned from this server. E-Mail %s for more information.",
/* 466 ERR_YOUWILLBEBANNED */	NULL,
/* 467 ERR_KEYSET */		":%s 467 %s %s :Channel key already set",
/* 468 ERR_ONLYSERVERSCANCHANGE */ ":%s 468 %s %s :Only servers can change that mode",
/* 469 */			NULL,
/* 470 */			NULL,
/* 471 ERR_CHANNELISFULL */	":%s 471 %s %s :Cannot join channel because it is full (+l)",
/* 472 ERR_UNKNOWNMODE */	":%s 472 %s %c :is an unknown mode char to me",
/* 473 ERR_INVITEONLYCHAN */	":%s 473 %s %s :Cannot join channel (invite only) [+i]",
/* 474 ERR_BANNEDFROMCHAN */	":%s 474 %s %s :Cannot join channel (banned) [+b]",
/* 475 ERR_BADCHANNELKEY */	":%s 475 %s %s :Cannot join channel (requires correct key) [+k]",
/* 476 ERR_BADCHANMASK */	":%s 476 %s %s :Bad channel mask",
/* 477 ERR_NEEDREGGEDNICK */	":%s 477 %s %s :Cannot join channel (registered nicknames only) [+R]",
/* 478 ERR_BANLISTFULL */	":%s 478 %s %s %s :Cannel ban/ignore list is full",
/* 479 ERR_FORWARDLISTFULL */	":%s 479 %s %s %s:%s :Channel forward list is full",
/* 480 */			NULL,
/* 481 ERR_NOPRIVILEGES */	":%s 481 %s :Permission Denied - You do not have the correct IRC Operator privileges",
/* 482 ERR_CHANOPRIVSNEEDED */	":%s 482 %s %s :You are not a channel operator",
/* 483 ERR_CANTKILLSERVER */	":%s 483 %s :You cannot kill a server!",
/* 484 */			NULL,
/* 485 */			NULL,
/* 486 */			NULL,
/* 487 */			NULL,
/* 488 */			NULL,
/* 489 */			NULL,
/* 490 */			NULL,
/* 491 ERR_NOOPERHOST */	":%s 491 %s :No O-Lines for your host",
/* 492 ERR_NOSERVICEHOST */	NULL,
/* 493 */			NULL,
/* 494 */			NULL,
/* 495 */			NULL,
/* 496 */			NULL,
/* 497 */			NULL,
/* 498 */			NULL,
/* 499 */			NULL,
/* 500 */			NULL,
/* 501 ERR_UNKNOWNMODEFLAG */	":%s 501 %s :Unknown MODE flag",
/* 502 ERR_USERSDONTMATCH */	":%s 502 %s :Cannot change other user's mode's",
/* 503 */			NULL,
/* 504 */			NULL,
/* 505 */			NULL,
/* 506 */			NULL,
/* 507 */			NULL,
/* 508 */			NULL,
/* 509 */			NULL,
/* 510 */			NULL,
/* 511 ERR_SILEISFULL */	":%s 511 %s %s :Your silence list is full",
/* 512 ERR_TOOMANYWATCH */	":%s 512 %s %s :Maximum size for WATCH List is 128 entries",
/* 513 ERR_NEEDPONG */		":%s 513 %s :To connect, type /QUOTE PONG %lX",
/* 514 */			NULL,
/* 515 */			NULL,
/* 516 */			NULL,
/* 517 */			NULL,
/* 518 */			NULL,
/* 519 */			NULL,
/* 520 */			NULL,
/* 521 ERR_LISTSYNTAX */	":%s 521 %s :Bad list syntax, type /QUOTE LIST ? or /RAW LIST ?",
/* 522 */			NULL,
/* 523 */			NULL,
/* 524 */			NULL,
/* 525 */			NULL,
/* 526 */			NULL,
/* 527 */			NULL,
/* 528 */			NULL,
/* 529 */			NULL,
/* 530 */			NULL,
/* 531 ERR_HTCTOOFAST */	":%s 531 %s :You are sending too fast, please wait %i seconds before sending new commands",
/* 532 */			NULL,
/* 533 */			NULL,
/* 534 */			NULL,
/* 535 */			NULL,
/* 536 */			NULL,
/* 537 */			NULL,
/* 538 */			NULL,
/* 539 */			NULL,
/* 540 */			NULL,
/* 541 */			NULL,
/* 542 */			NULL,
/* 543 */			NULL,
/* 544 */			NULL,
/* 545 */			NULL,
/* 546 */			NULL,
/* 547 */			NULL,
/* 548 */			NULL,
/* 549 */			NULL,
/* 550 */			NULL,
/* 551 */			NULL,
/* 552 */			NULL,
/* 553 */			NULL,
/* 554 */			NULL,
/* 555 */			NULL,
/* 556 */			NULL,
/* 557 */			NULL,
/* 558 */			NULL,
/* 559 */			NULL,
/* 560 */			NULL,
/* 561 */			NULL,
/* 562 */			NULL,
/* 563 */			NULL,
/* 564 */			NULL,
/* 565 */			NULL,
/* 566 */			NULL,
/* 567 */			NULL,
/* 568 */			NULL,
/* 569 */			NULL,
/* 570 */			NULL,
/* 571 */			NULL,
/* 572 */			NULL,
/* 573 */			NULL,
/* 574 */			NULL,
/* 575 */			NULL,
/* 576 */			NULL,
/* 577 */			NULL,
/* 578 */			NULL,
/* 579 */			NULL,
/* 580 */			NULL,
/* 581 */			NULL,
/* 582 */			NULL,
/* 583 */			NULL,
/* 584 */			NULL,
/* 585 */			NULL,
/* 586 */			NULL,
/* 587 */			NULL,
/* 588 */			NULL,
/* 589 */			NULL,
/* 590 */			NULL,
/* 591 */			NULL,
/* 592 */			NULL,
/* 593 */			NULL,
/* 594 */			NULL,
/* 595 */			NULL,
/* 596 */			NULL,
/* 597 */			NULL,
/* 598 */			NULL,
/* 599 */			NULL,
/* 600 RPL_LOGON */		":%s 600 %s %s %s %s %d :logged online",
/* 601 RPL_LOGOFF */		":%s 601 %s %s %s %s %d :logged offline",
/* 602 RPL_WATCHOFF */		":%s 602 %s %s %s %s %d :stopped watching",
/* 603 RPL_WATCHSTART */	":%s 603 %s :You have %d and are on %d WATCH entries",
/* 604 RPL_NOWON */		":%s 604 %s %s %s %s %d :is online",
/* 605 RPL_NOWOFF */		":%s 605 %s %s %s %s %d :is offline",
/* 606 RPL_WATCHLIST */		":%s 606 %s :%s",
/* 607 RPL_ENDOFWATCHLIST */	":%s 607 %s :End of WATCH %c",
/* 608 RPL_WATCHCLEAR */	":%s 608 %s :Your WATCH list is now empty",
/* 609 */			NULL,
/* 610 */			NULL,
/* 611 */			NULL,
/* 612 RPL_WHOWASHOST */	":%s 612 %s %s :was connecting from *@%s",
/* 613 RPL_WHOISHOST */		":%s 613 %s %s :is connecting from *@%s",
/* 614 RPL_WHOISMODES */	":%s 614 %s %s :is using modes %s",
/* 615 RPL_MAPMORE */		":%s 615 %s :%s%-*s --> *more*",
/* 616 */			NULL,
/* 617 */			NULL,
/* 618 */			NULL,
/* 619 */			NULL,
/* 620 */			NULL,
/* 621 */			NULL,
/* 622 */			NULL,
/* 623 */			NULL,
/* 624 */			NULL,
/* 625 */			NULL,
/* 626 */			NULL,
/* 627 */			NULL,
/* 628 */			NULL,
/* 629 */			NULL,
/* 630 */			NULL,
/* 631 */			NULL,
/* 632 */			NULL,
/* 633 */			NULL,
/* 634 */			NULL,
/* 635 */			NULL,
/* 636 */			NULL,
/* 637 */			NULL,
/* 638 */			NULL,
/* 639 */			NULL,
/* 640 */			NULL,
/* 641 */			NULL,
/* 642 */			NULL,
/* 643 */			NULL,
/* 644 */			NULL,
/* 645 */			NULL,
/* 646 */			NULL,
/* 647 */			NULL,
/* 648 */			NULL,
/* 649 */			NULL,
/* 650 */			NULL,
/* 651 */			NULL,
/* 652 */			NULL,
/* 653 */			NULL,
/* 654 */			NULL,
/* 655 */			NULL,
/* 656 */			NULL,
/* 657 */			NULL,
/* 658 */			NULL,
/* 659 */			NULL,
/* 660 */			NULL,
/* 661 */			NULL,
/* 662 */			NULL,
/* 663 */			NULL,
/* 664 */			NULL,
/* 665 */			NULL,
/* 666 */			NULL,
/* 667 */			NULL,
/* 668 */			NULL,
/* 669 */			NULL,
/* 670 */			NULL,
/* 671 */			NULL,
/* 672 */			NULL,
/* 673 */			NULL,
/* 674 */			NULL,
/* 675 */			NULL,
/* 676 */			NULL,
/* 677 */			NULL,
/* 678 */			NULL,
/* 679 */			NULL,
/* 680 */			NULL,
/* 681 */			NULL,
/* 682 */			NULL,
/* 683 */			NULL,
/* 684 */			NULL,
/* 685 */			NULL,
/* 686 */			NULL,
/* 687 */			NULL,
/* 688 */			NULL,
/* 689 */			NULL,
/* 690 */			NULL,
/* 691 */			NULL,
/* 692 */			NULL,
/* 693 */			NULL,
/* 694 */			NULL,
/* 695 */			NULL,
/* 696 */			NULL,
/* 697 */			NULL,
/* 698 */			NULL,
/* 699 */			NULL,
/* 700 ERR_BADNUMERIC */	":%s 700 %s :Numeric error!",
};

/* --------------------------------------------------------------------------- */

/* sends reply */
char	*getreply(int numeric)
{
	if ((numeric < 0 || numeric > 700) || !replies[numeric])
		return(replies[ERR_BADNUMERIC]);
	else
		return(replies[numeric]);
}

char	*rpl_str(int numeric)
{
	return getreply(numeric);
}
char	*err_str(int numeric)
{
	return getreply(numeric);
}
