/************************************************************************
 * IRC - Internet Relay Chat, src/s_vhost.c
 * Copyright (C) 2000 Alasdair McWilliam. RageIRCd, Inc.
 *
 * Contains code from UnrealIRCd - Copyright (C) The UnrealIRCd Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "dynconf2.h"
#include "config.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include "h.h"
#include "vhost.h"

#define AllocCpy(x, y) x = (char *)MyMalloc(strlen(y) + 1); strcpy(x, y)

aVhost	*vhosts = NULL;

int	add_vhost(login, passwd, vhost)
	char	*login;
	char	*passwd;
	char	*vhost;
{
	aVhost	*av;

	av = (aVhost *)MyMalloc(sizeof(aVhost));
	if (strlen(vhost) > (HOSTLEN - 1))
	{
		*(vhost + (HOSTLEN - 1)) = '\0';
	}
	AllocCpy(av->login, login);
	AllocCpy(av->passwd, passwd);
	AllocCpy(av->vhost, vhost);
	av->next = vhosts;
	av->prev = NULL;

	if (vhosts)
	{
		vhosts->prev = av;
	}
	vhosts = av;
}

aVhost	*del_vhost(av)
	aVhost	*av;
{
	aVhost	*x;
	aVhost	*z;

	for (x = vhosts; x; x = x->next)
	{
		if (x == av)
		{
			z = x->next;
			MyFree((char *)(av->login));
			MyFree((char *)(av->passwd));
			MyFree((char *)(av->vhost));

			if (x->prev)
			{
				x->prev->next = x->next;
			}
			else
			{
				vhosts = x->next;
			}

			if (x->next)
			{
				x->next->prev = x->prev;
			}
			MyFree((aVhost *) x);
			return z;
		}
	}
	return NULL;
}

int	load_vhost(file)
	char		*file;
{
	char		buf[2048];
	char		*x;
	char		*login, *passwd, *vhost;

	FILE		*fd = fopen(file, "r");
	if (!fd)
	{
		/* cannot find file */
		return -1;
	}

	while (fgets(buf, 2048, fd))
	{
		if ((buf[0] == '#') || (buf[0] == '/') || (buf[0] == '\0'))
		{
			/* comments */
			continue;
		}
		strip(buf);
		if ((buf[0] == '#') || (buf[0] == '/') || (buf[0] == '\0'))
		{
			/* comments */
			continue;
		}

		x = (char *)strtok((char *)buf, " ");
		if (strcmp("vhost", x) == 0)
		{
			/* set the login */
			login = (char *)strtok((char *)NULL, " ");
			if (!login)
			{
				/* no login */
				continue;
			}

			/* set the password */
			passwd = (char *)strtok((char *)NULL, " ");
			if (!passwd)
			{
				/* no password */
				continue;
			}

			/* set the vhost */
			vhost = (char *)strtok((char *)NULL, " ");
			if (!vhost)
			{
				/* no vhost */
				continue;
			}
			if (*vhost == '*')
			{
				continue;
			}

			/* add the vhost to the servers memory */
			add_vhost(login, passwd, vhost);
		}
	}
	fclose(fd);
	return 0;
}

/*
 * m_vhost - if user logs in correctly, users maskedhost is set to specified vhost
 *
 * parv[0] - sender previx
 * if (parc == 4)
 *      parv[1] = "ON"
 *      parv[2] = login
 *      parv[3] = password
 * if (parc == 2)
 *      parv[1] = "OFF"
 *
 * revamped -TD (December 2000)
 */
int	m_vhost(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	char		*what, *login, *passwd;
	FILE		*fd;
	aVhost		*av;

	fd = fopen(VHOST_FILE, "r");
	if (!fd)
	{
		sendto_one(sptr, err_str(ERR_UNKNOWNCOMMAND), me.name, parv[0], "VHOST");
		return 0;
	}
	fclose(fd);

	if (parc < 2)
	{
		sendto_one(sptr, ":%s NOTICE %s :*** vHost -- Syntax: /vhost <on/off> [login] [password]", me.name, parv[0]);
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "VHOST");
		return 0;
	}
	if (!MyClient(sptr))
		return 0;

	what = parv[1];
	if (!strcasecmp(what, "ON"))
	{
		if (parc < 4)
		{
			sendto_one(sptr, ":%s NOTICE %s :*** vHost -- Syntax: /vhost on <login> <password>", me.name, parv[0]);
			sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "VHOST");
			return 0;
		}

		login = parv[2];
		passwd = parv[3];

		for (av = vhosts; av; av = av->next)
		{
			/* check login */
			if (!strcmp(av->login, login))
			{
				/* login is ok, check password */
				if (!strcmp(av->passwd, passwd))
				{
					/* password is ok, vhost the user */
					sendto_ops("%s (%s@%s) has successfully logged into a vHost line (login: %s vhost: %s)", sptr->name,
						sptr->user->username, (IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host),
						av->login, av->vhost);
					if (MyClient(sptr))
					{
						sendto_serv_butone(&me, ":%s GLOBOPS :%s (%s@%s) has successfully logged into a vHost line"
							" (login: %s vhost: %s)", me.name, sptr->name, sptr->user->username,
							(IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host), av->login, av->vhost);
					}

					if (sptr->user->maskedhost)
					{
						MyFree(sptr->user->maskedhost);
					}
					sptr->user->maskedhost = MyMalloc(strlen(av->vhost) + 1);
					strcpy(sptr->user->maskedhost, av->vhost);
					sptr->umodes |= UMODE_MASK;

					sendto_serv_butone(cptr, ":%s SENDVHOST :%s", sptr->name, av->vhost);
					sendto_one(sptr, ":%s NOTICE %s :*** vHost -- [ON] Your hostname is now \'%s\'", me.name, sptr->name,
						sptr->user->maskedhost);
					return 0;
				}
				else
				{
					/* password is incorrect */
					sendto_ops("%s (%s@%s) failed to log into a vHost line (password incorrect)", sptr->name,
						sptr->user->username, (IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host));
					if (MyClient(sptr))
					{
						sendto_serv_butone(&me, ":%s GLOBOPS :%s (%s@%s) failed to log into a vHost line (password"
							" incorrect)", me.name, sptr->name, sptr->user->username,
							(IsMasked(sptr) ? sptr->user->maskedhost : sptr->user->host));
					}
					sendto_one(sptr, ":%s NOTICE %s :*** vHost -- Failed to log into a vHost line (password incorrect)",
						me.name, sptr->name);
					return 0;
				}
			}
		}
	}
	else if (!strcasecmp(what, "OFF"))
	{
		sptr->umodes |= UMODE_MASK;
		do_maskedhost(sptr);
		sendto_serv_butone(cptr, ":%s SENDVHOST *", sptr->name);

		sendto_one(sptr, ":%s NOTICE %s :*** vHost -- [OFF] Your hostname is now \'%s\'", me.name, sptr->name, sptr->user->maskedhost);
		return 0;
	}
	else
	{
		sendto_one(sptr, ":%s NOTICE %s :*** vHost -- Syntax: /vhost <on/off> [login] [password]", me.name, sptr->name);
		return 0;
	}
}

/*
 * sendvhost - sends the vhost change globally
 *
 *	parv[0] = sender prefix
 *	parv[1] = vhost or '*'
 *
 */
int	m_sendvhost(cptr, sptr, parc, parv)
	aClient		*cptr, *sptr;
	int		parc;
	char		*parv[];
{
	if (!IsServer(cptr))
		return 0;

	if (*parv[1] != '*')
	{
		if (sptr->user->maskedhost)
		{
			MyFree(sptr->user->maskedhost);
		}
		sptr->user->maskedhost = MyMalloc(strlen(parv[1]) + 1);
		strcpy(sptr->user->maskedhost, parv[1]);
		sptr->umodes |= UMODE_MASK;

		sendto_serv_butone(cptr, ":%s SENDVHOST :%s", sptr->name, parv[1]);
	}
	else
	{
		do_maskedhost(sptr);
		sptr->umodes |= UMODE_MASK;

		sendto_serv_butone(cptr, ":%s SENDVHOST *", sptr->name);
	}
	return 0;
}

void	stats_vhost(sptr)
	aClient		*sptr;
{
	aVhost		*av;
	char		*mask;

	for (av = vhosts; av; av = av->next)
	{
		mask = (BadPtr(av->vhost) ? "<NULL>" : av->vhost);
		sendto_one(sptr, ":%s %i %s :V %s %s", me.name, RPL_TEXT, sptr->name, av->login, mask);
	}
}

void	rehash_vhost(void)
{
	aVhost	*av, x;

	for (av = vhosts; av; av = av->next)
	{
		x.next = del_vhost(av);
		av = &x;
	}
	load_vhost(VHOST_FILE);
}
